//------------------------------------------------------------------------
// Having trouble dreaming up unique passwords that do not contain
// family names, pet names, streets you lived on, birth years, etc.
// The more passwords you have, the more of a pain in the neck it is to
// dream up new ones.
//
// This macro generates a unique password (or key) of any desired
// length containing a random mix of letters and numbers that are free
// of meaning or context and thus very hard to "crack".
//
// Using your own name or email address for your login "User ID"s is
// begging for trouble. To be extra safe, generate a key for your login
// "User ID"s as well.
//
// If you have a lot of passwords, you probably keep them in a password
// file.  It is advisable that your password file be encrypted.
//
// In addition to encryption, a variety of techniques may be used to
// disguise the meaning of passwords within a password file (i.e. which
// password belongs to which login, etc). We will not discuss those
// techniques here. Our main purpose is to help you generate secure
// passwords.
//
// This macro generates the new password directly into your password file
// (or any other file) at the current cursor position just as if you had
// typed it in.
//
// This macro requires 1 parameter:
//
//    Parameter1 = password-length
//
// The longer the password is, the more secure it will be.
//
// To generate a new password:
//
//    1) Edit your password file.
//    2) Type "NEWPSWD ...password-length..." in the primary command field.
//    3) Position the cursor where you want the new password generated.
//    4) Press "3270-Enter".
//
// Some letters/numbers may appear ambiguous in some fonts so we do not
// include them in the source letters and numbers.
//
//    Upper case letter "I", may look like number "1"
//    Lower case letter "l", may look like number "1"
//    Upper case letter "O", may look like number "0"
//    Lower case letter "o", may look like number "0"
//
//------------------------------------------------------------------------
void NewPswd(int argc, char *argv[]) {

  char *char_lower   = "abcdefghijkmnpqrstuvwxyz"; // no "l" or "o"
  char *char_upper   = "ABCDEFGHJKLMNPQRSTUVWXYZ"; // no "I" or "O"
  char *char_number  = "123456789";                // no "0"
  char *char_all; // catenation of letters and numbers
  char *char_assigned;

  char *new_pswd = ""; // init to NULL
  char *cur_char;
  char *cur_assigned;
  char *last_char;
  char *context;
  char *parm;

  int   random_range;
  int   min_pswd_len = 8;  // default MIN password length
  int   max_pswd_len = 32; // default MAX password length
  int   pswd_len;
  int   pos;
  int   rand_max;
  int   i, j;

  bool  any_error  = FALSE;
  bool  parm_error = FALSE;

  strcpy(char_all,char_number); // number 3 times to get approx 33% of key
  strcat(char_all,char_upper);
  strcat(char_all,char_number); // number 3 times to get approx 33% of key
  strcat(char_all,char_lower);
  strcat(char_all,char_number); // number 3 times to get approx 33% of key
  rand_max = strlen(char_all);
  char_assigned = strgenstr("0",rand_max); // string of "0"s equal in length to "char_all"
  rand_max -= 1;                // rand number will range from 0 to rand_max
  if (argc == 2) {
    context = SPFservice("query","context");
    if (context == "EDIT") {
      i = SPFservice("query","line_number");
      if (i == 0) {
        any_error = TRUE;
        Msg("NEWPSWD:  Cursor must be on a data line when this macro is invoked.");
        }
      else {
        i = SPFservice("query","col_number");
        if (i == 0) {
          any_error = TRUE;
          Msg("NEWPSWD:  Cursor must be on a data column when this macro is invoked.");
        }
      }
      }
    else {
      any_error = TRUE;
      Msg("NEWPSWD:  You must be in Edit on a file to invoke this macro.");
    }
    if (!any_error) {
      parm = argv[1];                  // get Parm1
      if (!StrIsDigit(parm)) {
        parm_error = TRUE;             // Error: Parm1 not numeric
      }
      if (!parm_error) {
        pswd_len = parm;               // get password length
        if (pswd_len < min_pswd_len) {
          parm_error = TRUE;           // Error: password too short
        }
        if (pswd_len > max_pswd_len) {
          parm_error = TRUE;           // Error: password too long
        }
      }
      if (!parm_error) {
        //------------------
        // generate password
        //------------------
        last_char = "~";               // init to char not used in passwords
        i = pswd_len;                  // number of chars to generate
        while (i) {
          pos = RandInRange(rand_max); // get letter in range 0 to rand_max
          pos += 1;                    // make number base 1
          cur_assigned = strgetsubstr(char_assigned,pos,1);
          if (cur_assigned == "0") {
            strovlsubstr(char_assigned,"1",pos);     // set char "assigned"
            cur_char = strgetsubstr(char_all,pos,1);
            strcat(new_pswd,cur_char); // catenate to password
            if (i > 0) --i;
          }
        } // while
        SPFservice("cmd",strcompose("DATA ",new_pswd));
        }
      else {
        parm_error = TRUE;
      }
    }
    }
  else {
    parm_error = TRUE;
  }
  if (parm_error) {
    Msg(
      strcompose(
      "NEWPSWD: This macro requires one parameter:\n\n"
      "     Parameter1 = password-length\n\n"
      "Minimum password length = ",
      min_pswd_len,
      "\nMaximum password length = ",
      max_pswd_len)
      );
  }

} // NewPswd
