//------------------------------------------------------------------------------
// This macro is intended to be used with email lists.  The email addresses must
// be in column 1.
//
// For each email address in the file, this macro left pads with blanks to force
// the atsign(@) to be in the same column as the rightmost atsign(@) in the set.
//
// For the sort, the macro makes all the lines lowercase.
//
// On completion of the re-alignment, the macro sorts the file on the email
// destination.
//
// After the sort, the pad blanks are eliminated.
//
// If you want to keep the original file "as is", use SAVEAS to put the result
// in a new file.
//
//------------------------------------------------------------------------------
void SortElist(void) {

  char *line_data;
  char *pad_blanks;
  char *pcmd;

  int   col_highwater = 0;
  int   linelen_highwater = 0;
  int   linelen;
  int   atsign_col;
  int   line_count; // number of lines in the file
  int   i, j;

  line_count = SPFservice("query","line_count");
  //---------------------------------------------
  // find rightmost atsign(@) col among all lines
  //---------------------------------------------
  i = 1;
  while (i <= line_count) {
    pcmd = strcompose("locate ",i); // locate to current line
    SPFservice("cmd",pcmd);
    line_data = SPFservice("query","line_data"); // get line data
    atsign_col = strfindlast(line_data,"@"); // find atsign(@) col for this line
    if (atsign_col > col_highwater)
      col_highwater = atsign_col; // remember rightmost atsign(@) col
    ++i;
  } // while
  //---------------------------------------------------------------
  // for each line, pad left with blanks to rightmost atsign(@) col
  //---------------------------------------------------------------
  i = 1;
  while (i <= line_count) {
    pcmd = strcompose("locate ",i); // locate to current line
    SPFservice("cmd",pcmd);
    line_data = SPFservice("query","line_data");
    atsign_col = strfindlast(line_data,"@");
    if (atsign_col < col_highwater) {
      j = col_highwater - atsign_col; // number of blanks needed
      pad_blanks = strgenstr(" ",j);
      strinssubstr(line_data,pad_blanks,1); // pad left with blanks
      SPFservice("set","line_data",line_data);
    }
    linelen = SPFservice("query","line_length");
    if (linelen > linelen_highwater) {
      linelen_highwater = linelen; // remember longest line
    }
    ++i;
  } // while
  //---------------------------------
  // make file all lowercase for sort
  //---------------------------------
  // locate top line, enter LCC line command
  pcmd = strcompose("locate ",1,";:lcc");
  SPFservice("cmd",pcmd);
  // locate bottom line, enter LCC line command
  pcmd = strcompose("locate ",line_count,";:lcc");
  SPFservice("cmd",pcmd);
  SPFservice("cmd","enter");
  //-------------------------------
  // sort file on email destination
  //-------------------------------
  pcmd = strcompose("sort ",col_highwater," ",linelen_highwater);
  SPFservice("cmd",pcmd);
  //------------------------------------
  // for each line, strip leading blanks
  //------------------------------------
  i = 1;
  while (i <= line_count) {
    pcmd = strcompose("locate ",i); // locate to current line
    SPFservice("cmd",pcmd);
    line_data = SPFservice("query","line_data");
    strstripleading(line_data," "); // strip leading blanks
    SPFservice("set","line_data",line_data);
    ++i;
  } // while
  SPFservice("cmd","top"); // return to top of file

} // SortElist
