//------------------------------------------------------------------------------
// Macro: DELDUPLINES
//
// Format:  DELDUPLINES  [start-column  end-column]
//
// Function:
//
//   If "start-column" and "end-column" are not specified, this macro sorts the
//   file, then deletes duplicate lines.
//    
//   If "start-column" and "end-column" are specified, the file is sorted on the
//   key field, then deletes lines with duplicate keys.
//
// NOTE: Most example macros provide trivial functionality. Their primary
// purpose is to show a variety of coding techniques.
//------------------------------------------------------------------------------
void delduplines(int argc, char *argv[]) {
 
  char *command;
  char *key1, *key2;
  char *line1, *line2;
  char *msg;
   
  int   delcount = 0;
  int   start_col = 0;
  int   end_col = 0;
  int   lines;
  int   key_length;
  int   i;
  
  BOOL  any_error = FALSE;
  BOOL  lines_match;
   
  if (argc > 1) {
    if (argc == 3) {
      start_col = argv[1];
      end_col = argv[2];
      if (start_col <= end_col) {
        key_length = end_col - start_col + 1;
        }
      else {
        any_error = TRUE;
        Msg("DELDUPLINES: 'start-col' must be equal or less than 'end-col'");
      }
      }
    else {
      any_error = TRUE;
      Msg("Correct form:  DELDUPLINES  start-col  end-col");
    }
  }
  if (!any_error) {
    lines = SPFservice("query","line_count");    // total lines in file
    if (lines > 1) {                             // enough lines to proceed?
      if (start_col > 0) {
        command = strcompose("sort ",start_col," ",end_col);
        SPFservice("cmd",command);               // sort by key
        }
      else {
        SPFservice("cmd","sort");                // sort whole lines
      }
      SPFservice("cmd","locate 1");              // position to first line
      line1 = SPFservice("query","line_data");   // get 1st line of potential block
      i = 2;                                     // 2nd line number
      while (i <= lines) {
        command = StrCompose("locate ",i);
        SPFservice("cmd",command);
        line2 = SPFservice("query","line_data"); // get 2nd line of potential block
        lines_match = FALSE;
        if (start_col > 0) {
          key1 = strgetsubstr(line1,start_col,key_length); // get key from line 1
          key2 = strgetsubstr(line2,start_col,key_length); // get key from line 2
          if (key1 == key2) {                    // keys match?
            lines_match = TRUE;                  // yes  
          }
          }
        else {
          if (line1 == line2) {                  // whole lines match?
            lines_match = TRUE;                  // yes  
          }
        }
        if (lines_match) {
          SPFservice("cmd",":d");                // delete second line of pair
          --lines;                               // decr total line count
          ++delcount;                            // incr deleted count
          }
        else {
          // lines don't match, start new pair, continue
          line1 = line2;
          ++i;
        }
      } // while
      SPFservice("cmd","top");                   // re-position to top of file
      msg = StrCompose("Deleted ",delcount," duplicate lines.");
      Msg(msg);
      }
    else {
      Msg("DELDUPLINES: file must have at least two lines for comparison.");
    }
  }

} // delduplines
