//--------------------------------------------------------------------
// This macro deletes all empty lines in a RECFM=V file. It assumes
// that you have "Strip trailing blanks" set to ON in the file profile
// for the type in question. This macro does not work for RECFM=F
// files because their blank lines are not empty, they contain blanks
// in all data positions. To handle RECFM=F you would have to retrieve
// the line data, scan full length for non-blank, then delete if the
// line was all blank. This method would of course be much slower.
//--------------------------------------------------------------------
void delemptylines(void) {
 
  char *cmd;
  char *msg;
  int   i, j;
  int   line_length;
  int   lines_deleted = 0;
   
  i = 1;
  j = SPFservice("query","line_count");
  if (j > 0) {
    while (i < j) {
      cmd = StrCompose("locate ",i); // synthesize locate command
      SPFservice("cmd",cmd);         // locate to next line
      line_length = SPFservice("query","line_length"); // get line length
      if (line_length == 0) {
        //------------------
        // delete empty line
        //------------------
        SPFservice("cmd",":d");
        j = SPFservice("query","line_count"); // update line count
        ++lines_deleted;             // increment lines deleted count
        }
      else {
        ++i;                         // go to next line
      }
    } // while
  }
  SPFservice("cmd","top");           // position to top of file
  if (lines_deleted) {
    msg = StrCompose(lines_deleted," empty lines deleted.");
    }
  else {
    msg = "No empty lines in this file.";
  }
  Msg(msg);
 
} // delemptylines
