//-----------------------------------------------------------------------------
// Macro: PHONEBK
//
// Form:  PHONEBK
//
//   No parameters.
//
// Function:
//
//   Provides a simple database for keeping address and phone number
//   information.
//
// NOTE: Most example macros provide trivial functionality. Their primary
// purpose is to show a variety of coding techniques.
//-----------------------------------------------------------------------------
 
//----------------------------------------------
// subscript values for phone book record fields
//----------------------------------------------
int const LASTNAME  = 0;
int const FIRSTNAME = 1;
int const PHONE1    = 2;
int const PHONE2    = 3;
int const FAX       = 4;
int const EMAIL     = 5;
int const TITLE     = 6;
int const COMPANY   = 7;
int const STREET    = 8;
int const CITY      = 9;
int const STATE     = 10;
int const ZIP       = 11;
int const COUNTRY   = 12;
int const NOTE      = 13;
   
//------------
// global vars
//------------
 
char *g_database[];          // array of records
 
int   g_records = 0;         // number of records in the database
 
BOOL  g_modified = FALSE;    // TRUE = one or more recs modified
 
//--------
// PHONEBK
//--------
void phonebk(void) {
 
  //-----------
  // local vars
  //-----------
  char *alpha[];             // array to hold alphabet
  char *buf;                 // general purpose buffer
  char *filespec;            // database file spec
  char *entry_group;
  char *fields[];            // array to hold parsed record fields
  char *firstchar;
  char *letter;
  char *NameEntry;
  char *NameSelect;
  char *RCSTR;               // return code from read
  char *recbuf;              // buffer to receive input record
  char *varname;
  char *dialog_rc;
   
  char *BtnPushed;
   
  int   field_count;
  int   hfile;               // file handle
  int   i;
  int   matches;             // number of recs which match selected letter
  int   sel_ix;
  int   sel_count;
   
  BOOL  any_error = FALSE;
  BOOL  cancel = FALSE;
   
  filespec = SPFservice("vget","Phonebk1.FileSpec");
  if (filespec == "") {
    dialog_rc = SPFservice("dialog","phonebk1"); // present file spec dialog
    switch (dialog_rc) {

      case "OK":
        BtnPushed = SPFservice("vget","phonebk1.BtnPushed"); // get button state
        break;

      case "HELP":
        cancel = TRUE;
        Msg("HELP not available for this dialog.");
        break;

      case "CANCEL":
        cancel = TRUE;
        break;

      case "ERROR":
        any_error = TRUE;
        break;

    } // switch
  }
  if (!any_error && !cancel) {
    //-------------------------
    // open phone book database
    //-------------------------
    filespec = SPFservice("vget","Phonebk1.FileSpec");
    if (filespec != "") {
      hfile = FileOpen(filespec,"r");     // try to open existing phone book file
      if (hfile == NULL) {
        hfile = FileOpen(filespec,"w");   // can't open existing, create new database
        if (hfile > 0) {
          FileClose(hfile);               // create successful, close it
          hfile = FileOpen(filespec,"r"); // re-open in READ mode 
        }
      }
      if (hfile > 0) {
        // make file spec persistent for future invokations of dialog
        SPFservice("vpersist","Phonebk1.FileSpec");
        }
      else {
        any_error = TRUE;
        buf = StrCompose("Unable to create ",filespec,", can not continue");
        Msg(buf);
      }
      }
    else {
      cancel = TRUE;
    }
  }
  if (!any_error && !cancel) {
    //---------------------------------
    // read phone book database records
    //---------------------------------
    // RC gets return code, "recbuf" gets record
    RCSTR = FileRecRead(hfile,recbuf);
    while (RCSTR == "OK") {               // read till EOF or ERROR
      g_database[g_records] = recbuf;     // copy record to database array
      ++g_records;                        // incr record count
      RCSTR = FileRecRead(hfile,recbuf);  // read next record
    } // while
    //-----------
    // close file
    //-----------
    FileClose(hfile);
    if (RCSTR != "EOF") {
      any_error = TRUE;
      buf = StrCompose("Error reading ",filespec,", RC=",RCSTR);
      Msg(buf);
    }
  }
  if (!any_error && !cancel) {
    //-----------------------------
    // load "alpha" array with A..Z
    //-----------------------------
    StrParse(alpha,"A B C D E F G H I J K L M N O P Q R S T U V W X Y Z"," ");
  }
  while (!any_error && !cancel) {
    letter = ""; // reset
    dialog_rc = SPFservice("dialog","phonebk2");          // present "letter" dialog
    switch (dialog_rc) {

      case "OK":
        BtnPushed = SPFservice("vget","phonebk2.BtnPushed"); // get button state
        strupr(BtnPushed);
        break;

      case "HELP":
        cancel = TRUE;
        Msg("HELP not available for this dialog.");
        break;

      case "CANCEL":
        cancel = TRUE;
        break;

      case "ERROR":
        any_error = TRUE;
        break;

    } // switch
    if (!any_error && !cancel) {
      switch (BtnPushed) {
       
        case "NEWENTRY":
          PhoneEntryNew();
          break;

        case "A","B","C","D","E","F","G","H","I","J":
          letter = BtnPushed;
          break;

        case "K","L","M","N","O","P","Q","R","S","T":
          letter = BtnPushed;
          break;

        case "U","V","W","X","Y","Z":
          letter = BtnPushed;
          break;

      } // switch
      if (letter) {
        //-----------------------------------------------------
        // find records in database which match selected letter
        //-----------------------------------------------------
        matches = 0;                         // reset
        i = 0;                               // first record index
        while (i < g_records) {              // scan records
          firstchar = StrGetSubstr(g_database[i],2,1); // get 1st char of last name
          if (firstchar == letter) {         // chars match?
            //-------------------------------------------
            // found record which matches selected letter
            //-------------------------------------------
            ++matches;                       // incr match count
            // load array from comma delimited format record
            field_count = StrParse(fields,g_database[i],",");
            if (field_count >= 2) {
              NameEntry = StrCompose(fields[LASTNAME],",",
                            fields[FIRSTNAME]); // compose list entry
              varname = StrCompose("phonebk3.List.",matches); // make var name
              SPFservice("vput",varname,NameEntry);    // put name in dialog list
            }
          }
          if (firstchar > letter)
            i = g_records;  // end of qualified entries, end loop
          else
            ++i;            // index to next record
        } // while
        if (matches) {
          //-----------------------------------------
          // let user select a specific entry to edit
          //-----------------------------------------
          SPFservice("vput","phonebk3.List.0",matches); // set number of entries
          SPFservice("vput","phonebk3.List.Select",1);  // select 1st entry by default
          dialog_rc = SPFservice("dialog","phonebk3");  // show matching records
          switch (dialog_rc) {

            case "OK":
              BtnPushed = SPFservice("vget","phonebk3.BtnPushed"); // get button state
              strupr(BtnPushed);
              break;

            case "HELP":
              cancel = TRUE;
              Msg("HELP not available for this dialog.");
              break;

            case "CANCEL":
              cancel = TRUE;
              break;

            case "ERROR":
              any_error = TRUE;
              break;

          } // switch
          if (!any_error && !cancel) {
            sel_count = SPFservice("vget","phonebk3.List.Reply.0"); // get number of selected entries
            if (sel_count == 1) {             // name selected?
              //---------------
              // entry selected
              //---------------
              NameSelect = SPFservice("vget","phonebk3.List.Reply.1"); // get selected name
              BtnPushed = SPFservice("vget","phonebk3.BtnPushed"); // get button state
              strupr(BtnPushed);
              if (BtnPushed == "DELETE")
                PhoneEntryDelete(NameSelect); // delete selected entry
              else
                PhoneEntryEdit(NameSelect);   // edit selected entry
            }
            }
          else {
            cancel = FALSE; // reset to allow outer loop to continue
          }
          }
        else {
          buf = StrCompose("No entries for letter '",letter,"'");
          Msg(buf);
        }
      }
    }
  } // while
  if (!any_error && g_modified) {
    //------------------------
    // write modified database
    //------------------------
    hfile = FileOpen(filespec,"w");        // open database
    if (hfile > 0) {
      i = 0;                               // 1st rec index
      while (i < g_records) {              // more recs?
        FileRecWrite(hfile,g_database[i]); // write record
        ++i;                               // next rec index
      } // while
      FileClose(hfile);
      }
    else {
      buf = StrCompose("Unable to write modified database ",filespec);
      Msg(buf);
    }
  }
   
} // phonebk
 
//--------------
// PhoneEntryNew
//--------------
void PhoneEntryNew(void) {
 
  char *BtnPushed;
  char *buf;
  char *fields[];   // array of fields from dialog
  char *fields2[];  // array of fields from one record
  char *firstchar;
  char *firstname;
  char *lastname;
  char *letter;
  char *separator;
  char *dialog_rc;
   
  int   commacol;
  int   hfile;
  int   ins_ix;
  int   i;
   
  BOOL  any_error = FALSE;
  BOOL  cancel = FALSE;
   
  SPFservice("vput","phonebk4.LastName", "");
  SPFservice("vput","phonebk4.FirstName","");
  SPFservice("vput","phonebk4.Phone1",   "");
  SPFservice("vput","phonebk4.Phone2",   "");
  SPFservice("vput","phonebk4.Fax",      "");
  SPFservice("vput","phonebk4.Email",    "");
  SPFservice("vput","phonebk4.Title",    "");
  SPFservice("vput","phonebk4.Company",  "");
  SPFservice("vput","phonebk4.Street",   "");
  SPFservice("vput","phonebk4.City",     "");
  SPFservice("vput","phonebk4.State",    "");
  SPFservice("vput","phonebk4.PostCode", "");
  SPFservice("vput","phonebk4.Country",  "");
  SPFservice("vput","phonebk4.Note",     "");
  //-----------------------
  // present the NEW dialog
  //-----------------------
  dialog_rc = SPFservice("dialog","phonebk4");
  switch (dialog_rc) {

    case "OK":
      BtnPushed = SPFservice("vget","phonebk4.BtnPushed"); // get button state
      strupr(BtnPushed);
      break;

    case "HELP":
      cancel = TRUE;
      Msg("HELP not available for this dialog.");
      break;

    case "CANCEL":
      cancel = TRUE;
      break;

    case "ERROR":
      any_error = TRUE;
      break;

  } // switch
  if (!any_error && !cancel) {
    //--------------------------
    // retrieve new field values
    //--------------------------
    g_modified = TRUE;  // set "database modified"
     
    fields[LASTNAME]  = SPFservice("vget","phonebk4.LastName");
    fields[FIRSTNAME] = SPFservice("vget","phonebk4.FirstName");
    fields[PHONE1]    = SPFservice("vget","phonebk4.Phone1");
    fields[PHONE2]    = SPFservice("vget","phonebk4.Phone2");
    fields[FAX]       = SPFservice("vget","phonebk4.Fax");
    fields[EMAIL]     = SPFservice("vget","phonebk4.Email");
    fields[TITLE]     = SPFservice("vget","phonebk4.Title");
    fields[COMPANY]   = SPFservice("vget","phonebk4.Company");
    fields[STREET]    = SPFservice("vget","phonebk4.Street");
    fields[CITY]      = SPFservice("vget","phonebk4.City");
    fields[STATE]     = SPFservice("vget","phonebk4.State");
    fields[ZIP]       = SPFservice("vget","phonebk4.PostCode");
    fields[COUNTRY]   = SPFservice("vget","phonebk4.Country");
    fields[NOTE]      = SPFservice("vget","phonebk4.Note");
     
    // find correct insert point in database based on ascending sort order 
    letter = StrGetSubstr(fields[LASTNAME],1,1); // get 1st letter of last name
    ins_ix = 0;                                  // 1st rec index
    while (!cancel && ins_ix < g_records) {      // more recs?
      firstchar = StrGetSubstr(g_database[ins_ix],2,1); // get first char of last name
      if (firstchar == letter) {
        //---------------------
        // first letter matches
        //---------------------
        // load array from comma delimited format record
        StrParse(fields2,g_database[ins_ix],",");
        if (fields[LASTNAME] == fields2[LASTNAME]) {
          // last name matches
          if (fields[FIRSTNAME] < fields2[FIRSTNAME]) cancel = TRUE; // new first < current first, insert here
          }
        else {
          if (fields[LASTNAME] < fields2[LASTNAME]) cancel = TRUE; // new last < current last, insert here
        }
      }
      if (!cancel) {
        if (firstchar > letter)
          cancel = TRUE; // end of qualified entries, use curent index
        else
          ++ins_ix;    // next rec index
      }
    } // while
    if (ins_ix < g_records && g_records) {
      //-------------------------------------------------------
      // move existing records after insert point down one slot
      //-------------------------------------------------------
      i = g_records - 1; // index to last actual record
      while (i >= ins_ix) {
        g_database[i+1] = g_database[i]; // move record down one slot
        --i;                             // step index
      } // while
    }
    ++g_records;       // increment record count
    //--------------------------------
    // insert new record into database
    //--------------------------------
    separator = '","'; // "comma delimited format" separator
    g_database[ins_ix] = StrCompose(
      '"', // first quote
       
      fields[LASTNAME],  separator,
      fields[FIRSTNAME], separator,
      fields[PHONE1],    separator,
      fields[PHONE2],    separator,
      fields[FAX],       separator,
      fields[EMAIL],     separator,
      fields[TITLE],     separator,
      fields[COMPANY],   separator,
      fields[STREET],    separator,
      fields[CITY],      separator,
      fields[STATE],     separator,
      fields[ZIP],       separator,
      fields[COUNTRY],   separator,
      fields[NOTE],
       
      '"', // last quote
      );
  }
   
} // PhoneEntryNew
 
//---------------
// PhoneEntryEdit
//---------------
void PhoneEntryEdit(char *WholeName) {
 
  char *BtnPushed;
  char *fields[];
  char *msg;
  char *record;
  char *separator;
  char *temp_name;
  char *dialog_rc;
   
  int   hfile;
  int   i;
   
  BOOL  any_error = FALSE;
  BOOL  cancel = FALSE;
   
  i = PhoneEntryFind(WholeName); // find matching entry number (returns base 1)
  if (i > 0) {
    //----------------------
    // matching record found
    //----------------------
    --i; // make index base 0
     
    // load array from comma delimited format record
    StrParse(fields,g_database[i],",");
     
    SPFservice("vput","phonebk4.LastName", fields[LASTNAME]);
    SPFservice("vput","phonebk4.FirstName",fields[FIRSTNAME]);
    SPFservice("vput","phonebk4.Phone1",   fields[PHONE1]);
    SPFservice("vput","phonebk4.Phone2",   fields[PHONE2]);
    SPFservice("vput","phonebk4.Fax",      fields[FAX]);
    SPFservice("vput","phonebk4.Email",    fields[EMAIL]);
    SPFservice("vput","phonebk4.Title",    fields[TITLE]);
    SPFservice("vput","phonebk4.Company",  fields[COMPANY]);
    SPFservice("vput","phonebk4.Street",   fields[STREET]);
    SPFservice("vput","phonebk4.City",     fields[CITY]);
    SPFservice("vput","phonebk4.State",    fields[STATE]);
    SPFservice("vput","phonebk4.PostCode", fields[ZIP]);
    SPFservice("vput","phonebk4.Country",  fields[COUNTRY]);
    SPFservice("vput","phonebk4.Note",     fields[NOTE]);
     
    //-----------------------------
    // present PHONEBK entry dialog
    //-----------------------------
    dialog_rc = SPFservice("dialog","phonebk4");
    switch (dialog_rc) {

      case "OK":
        BtnPushed = SPFservice("vget","phonebk4.BtnPushed");  // get button state
        strupr(BtnPushed);
        break;

      case "HELP":
        cancel = TRUE;
        Msg("HELP not available for this dialog.");
        break;

      case "CANCEL":
        cancel = TRUE;
        break;

      case "ERROR":
        any_error = TRUE;
        break;

    } // switch
    if (!any_error && !cancel) {
      //----------------------
      // retrieve field values
      //----------------------
      fields[LASTNAME]  = SPFservice("vget","phonebk4.LastName");
      fields[FIRSTNAME] = SPFservice("vget","phonebk4.FirstName");
      fields[PHONE1]    = SPFservice("vget","phonebk4.Phone1");
      fields[PHONE2]    = SPFservice("vget","phonebk4.Phone2");
      fields[FAX]       = SPFservice("vget","phonebk4.Fax");
      fields[EMAIL]     = SPFservice("vget","phonebk4.Email");
      fields[TITLE]     = SPFservice("vget","phonebk4.Title");
      fields[COMPANY]   = SPFservice("vget","phonebk4.Company");
      fields[STREET]    = SPFservice("vget","phonebk4.Street");
      fields[CITY]      = SPFservice("vget","phonebk4.City");
      fields[STATE]     = SPFservice("vget","phonebk4.State");
      fields[ZIP]       = SPFservice("vget","phonebk4.PostCode");
      fields[COUNTRY]   = SPFservice("vget","phonebk4.Country");
      fields[NOTE]      = SPFservice("vget","phonebk4.Note");

      switch (BtnPushed) {
        case "PRINT":
          //------------------------------------
          // write the entry to a temporary file
          //------------------------------------
          temp_name = "temp.txt"; // make it a var in case we need to change it
          hfile = FileOpen(temp_name,"w");
          if (hfile > 0) {
            record = StrCompose(fields[FIRSTNAME]," ",fields[LASTNAME]);
            FileRecWrite(hfile,record);
            if (fields[TITLE])   FileRecWrite(hfile,fields[TITLE]);
            if (fields[COMPANY]) FileRecWrite(hfile,fields[COMPANY]);
            if (fields[STREET])  FileRecWrite(hfile,fields[STREET]);
            record = "";
            if (fields[CITY]) {
              record = fields[CITY];
              if (fields[STATE]) {
                record = StrCompose(record,", ",fields[STATE]);
                if (fields[ZIP]) {
                  record = StrCompose(record,", ",fields[ZIP]);
                }
              }
            }
            if (record) {
              // write CITY, STATE, ZIP
              FileRecWrite(hfile,record);
            }
            if (fields[PHONE1]) {
              record = StrCompose("PHONE1: ",fields[PHONE1]);
              FileRecWrite(hfile,record);
            }
            if (fields[PHONE2]) {
              record = StrCompose("PHONE2: ",fields[PHONE2]);
              FileRecWrite(hfile,record);
            }
            if (fields[FAX]) {
              record = StrCompose("FAX:    ",fields[FAX]);
              FileRecWrite(hfile,record);
            }
            if (fields[EMAIL]) {
              record = StrCompose("EMAIL:  ",fields[EMAIL]);
              FileRecWrite(hfile,record);
            }
            FileClose(hfile);
            FilePrint(temp_name);
            }
          else {
            msg = StrCompose("Unable to open ",temp_name," for write");
            Msg(msg);
          }
          break;

        case "CHANGE":
          //------------------------------------
          // put new values into original record
          //------------------------------------
          g_modified = TRUE; // set "database modified"
          separator = '","'; // "comma delimited format" separator
          g_database[i] = StrCompose(
            '"', // first quote

            fields[LASTNAME],  separator,
            fields[FIRSTNAME], separator,
            fields[PHONE1],    separator,
            fields[PHONE2],    separator,
            fields[FAX],       separator,
            fields[EMAIL],     separator,
            fields[TITLE],     separator,
            fields[COMPANY],   separator,
            fields[STREET],    separator,
            fields[CITY],      separator,
            fields[STATE],     separator,
            fields[ZIP],       separator,
            fields[COUNTRY],   separator,
            fields[NOTE],

            '"', // last quote
            );
          break;

      } // switch
    }
  }
   
}  // PhoneEntryEdit
 
//-----------------
// PhoneEntryDelete
//-----------------
void PhoneEntryDelete(char *WholeName) {
 
  int  i;
   
  i = PhoneEntryFind(WholeName); // find matching entry
  // NOTE: 0 = not-found, 1..N = record number, base 1
  if (i > 0) { // found?
    //------------------------------------------------------------
    // delete record by shifting all following records up one slot
    // in the database array
    //------------------------------------------------------------
    --i; // make record number base 1;
    while (i < g_records) {
      g_database[i] = g_database[i+1];
      ++i;
    }
    --g_records; // decrement record count
    g_modified = TRUE;
  }
   
}  // PhoneEntryDelete
 
//---------------
// PhoneEntryFind
//---------------
int PhoneEntryFind(char *WholeName) {
 
  // RETURNS one of:  1) base 1 record number of matching record
  //                  2) 0 if no match
   
  char *fields[];  // array to hold fields from one record
  char *firstchar;
  char *firstname;
  char *lastname;
  char *letter;
   
  int   commacol;
  int   field_count;
  int   i, j;
   
  BOOL  done;
  BOOL  found;
   
  commacol = StrFindSubstr(WholeName,",",1);  // find end of last name
  if (commacol > 1) {
    i = commacol - 1;                         // length of last name
    lastname = StrGetSubstr(WholeName,1,i);   // extract last name
    i = commacol + 1;                         // col after comma
    j = strlen(WholeName);                    // get length of whole name
    j -= commacol;                            // number of chars in first name
    firstname = StrGetSubstr(WholeName,i,j);  // extract first name (to end of string)
    letter = StrGetSubstr(lastname,1,1);      // first char of last name
  }
  done  = FALSE;                              // reset
  found = FALSE;                              // reset
  i = 0;                                      // 1st rec index
  while (!done && !found && i < g_records) {  // more recs?
    firstchar = StrGetSubstr(g_database[i],2,1); // get first char of last name
    if (firstchar == letter) {
      //---------------------
      // first letter matches
      //---------------------
      // load array from comma delimited format record
      field_count = StrParse(fields,g_database[i],",");
      if ( field_count >= 2
        && fields[LASTNAME]  == lastname
        && fields[FIRSTNAME] == firstname ) {
        //-------------------
        // whole name matches
        //-------------------
        found = TRUE;
      }
    }
    if (!found) {
      if (firstchar > letter)
        done = TRUE; // end of qualified entries, force end of file
      else
        ++i;         // next rec index
    }
  } // while
  if (found)
    ++i;   // return record number, base 1
  else
    i = 0; // return "not found" indicator
  return(i);
   
}  // PhoneEntryFind
