//------------------------------------------------------------------------
// Macro: SPELLCHECK
//
// Form:  SPELLCHECK
//
//   No parameters.
//
// Function:
//
//   This macro spell checks each word (all alpha characters) in the
//   currently edited file against the user specified dictionary. If a
//   word is not in the dictionary it pops up a dialog offering you the
//   option of keying in the correct spelling or adding the word to the
//   dictionary. By default it spellchecks the current selection. If there
//   is no current selection, it spell checks the entire file.
//
// Dictionary:
//
//   A basic dictionary is provided in
//
//      C:\spfse-install-directory\MACROS\SPELLCHECK.TXT
//
//   Copy it to another directory like C:\SPELLCHK. If you use it in
//   place, it will be overlaid when you upgrade SPF and you would loose
//   any words you have added.
//
// NOTE: This macro is neither fast nor elegant. Its primary purpose is to
// show a variety of macro coding techniques.
//------------------------------------------------------------------------

int const WRITE_THRESHOLD = 70;

//------------
// global vars
//------------
char *g_ignore_words[];
char *g_recbuf = "";
int   g_reclen = 0;
int   g_ignore_count = 0; // number of ignore words
int   g_hfile        = 0; // dictionary file handle
BOOL  g_end          = FALSE;
BOOL  g_modified     = FALSE;

char *a_words[];
char *b_words[];
char *c_words[];
char *d_words[];
char *e_words[];
char *f_words[];
char *g_words[];
char *h_words[];
char *i_words[];
char *j_words[];
char *k_words[];
char *l_words[];
char *m_words[];
char *n_words[];
char *o_words[];
char *p_words[];
char *q_words[];
char *r_words[];
char *s_words[];
char *t_words[];
char *u_words[];
char *v_words[];
char *w_words[];
char *x_words[];
char *y_words[];
char *z_words[];

int   a_count = 0;
int   b_count = 0;
int   c_count = 0;
int   d_count = 0;
int   e_count = 0;
int   f_count = 0;
int   g_count = 0;
int   h_count = 0;
int   i_count = 0;
int   j_count = 0;
int   k_count = 0;
int   l_count = 0;
int   m_count = 0;
int   n_count = 0;
int   o_count = 0;
int   p_count = 0;
int   q_count = 0;
int   r_count = 0;
int   s_count = 0;
int   t_count = 0;
int   u_count = 0;
int   v_count = 0;
int   w_count = 0;
int   x_count = 0;
int   y_count = 0;
int   z_count = 0;

int   all_count = 0; // total words in dictionary

//-----------
// spellcheck
//-----------
void spellcheck(void) {

  char *filespec;
  char *backup_spec;
  char *words[];          // parsed dictionary words from one record
  char *original_words[]; // parsed document words from one record
  char *upcase_words[];   // parsed document words from one record
  char *record;           // dictionary record
  char *dialog_rc;

  char *msg;
  char *cmd;
  char *line_data, *normalized_data;
  char *word1, *word2;
  char *parsestr;
  char *RC_STR;      // file I/O return string
  char *first_char, *last_char;

  int   linenum, start_linenum, end_linenum;
  int   task_count;
  int   word_count;  // word count from one record
  int   wordix;
  int   curcol;
  int   wordlen;
  int   i;

  BOOL  any_error = FALSE;
  BOOL  cancel = FALSE;
  BOOL  done;
  BOOL  check;

  if (SPFservice("query","task_count")) {
    filespec = SPFservice("vget","SpellFile.FileSpec");
    if (filespec == "") {
      //-------------------------
      // get spellcheck file name
      //-------------------------
      dialog_rc = SPFservice("dialog","spellfile"); // present file spec dialog
      switch (dialog_rc) {

        case "OK":
          break;

        case "HELP":
          cancel = TRUE;
          Msg("HELP not available for this dialog.");
          break;

        case "CANCEL":
          cancel = TRUE;
          break;

        case "ERROR":
          any_error = TRUE;
          break;

      } // switch
      if (!any_error && !cancel) {
        filespec = SPFservice("vget","SpellFile.FileSpec");
      }
    }
    if (filespec != "") {
      //----------------
      // load dictionary
      //----------------
      g_hfile = FileOpen(filespec,"r"); // open dictionary file
      if (g_hfile) {
        done = FALSE;
        while (!done) {
          RC_STR = FileRecRead(g_hfile,record); // read next dictionary record
          if (RC_STR == "OK") {
            // process dictionary record
            word_count = StrParse(words,record," "); // parse into separate words
            i = 0; // first new word index
            while (i < word_count) {
              // append each word to appropriate list
              first_char = StrGetSubstr(words[i],1,1);
              ++all_count; // incr total word count
              switch (first_char) {
                case "A":
                  a_words[a_count] = words[i];
                  ++a_count;
                  break;
                case "B":
                  b_words[b_count] = words[i];
                  ++b_count;
                  break;
                case "C":
                  c_words[c_count] = words[i];
                  ++c_count;
                  break;
                case "D":
                  d_words[d_count] = words[i];
                  ++d_count;
                  break;
                case "E":
                  e_words[e_count] = words[i];
                  ++e_count;
                  break;
                case "F":
                  f_words[f_count] = words[i];
                  ++f_count;
                  break;
                case "G":
                  g_words[g_count] = words[i];
                  ++g_count;
                  break;
                case "H":
                  h_words[h_count] = words[i];
                  ++h_count;
                  break;
                case "I":
                  i_words[i_count] = words[i];
                  ++i_count;
                  break;
                case "J":
                  j_words[j_count] = words[i];
                  ++j_count;
                  break;
                case "K":
                  k_words[k_count] = words[i];
                  ++k_count;
                  break;
                case "L":
                  l_words[l_count] = words[i];
                  ++l_count;
                  break;
                case "M":
                  m_words[m_count] = words[i];
                  ++m_count;
                  break;
                case "N":
                  n_words[n_count] = words[i];
                  ++n_count;
                  break;
                case "O":
                  o_words[o_count] = words[i];
                  ++o_count;
                  break;
                case "P":
                  p_words[p_count] = words[i];
                  ++p_count;
                  break;
                case "Q":
                  q_words[q_count] = words[i];
                  ++q_count;
                  break;
                case "R":
                  r_words[r_count] = words[i];
                  ++r_count;
                  break;
                case "S":
                  s_words[s_count] = words[i];
                  ++s_count;
                  break;
                case "T":
                  t_words[t_count] = words[i];
                  ++t_count;
                  break;
                case "U":
                  u_words[u_count] = words[i];
                  ++u_count;
                  break;
                case "V":
                  v_words[v_count] = words[i];
                  ++v_count;
                  break;
                case "W":
                  w_words[w_count] = words[i];
                  ++w_count;
                  break;
                case "X":
                  x_words[x_count] = words[i];
                  ++x_count;
                  break;
                case "Y":
                  y_words[y_count] = words[i];
                  ++y_count;
                  break;
                case "Z":
                  z_words[z_count] = words[i];
                  ++z_count;
                  break;
              } // switch
              ++i; // index to next new word
            } // while
            }
          else {
            // EOF or ERROR
            done = TRUE;
            FileClose(g_hfile); // close dictionary file
          }
        } // while
      }
      }
    else {
      cancel = TRUE;
    }
    if (!any_error && !cancel) {
      //
      // If you are curious about the dictionary size, you can
      // un-comment the following two lines.
      //
      // msg = StrCompose("Total words in dictionary: ",all_count);
      // Msg(msg);
      //
      start_linenum = SPFservice("query","select_start_line");
      if (start_linenum) {
        end_linenum   = SPFservice("query","select_end_line");
        }
      else {
        start_linenum = 1;
        end_linenum   = SPFservice("query","line_count");
      }
      linenum = start_linenum;
      while (!g_end && linenum <= end_linenum) {
        //-----------------
        // locate next line
        //-----------------
        cmd = StrCompose("locate ",linenum);
        SPFservice("cmd",cmd);
        line_data = SPFservice("query","line_data");
        if (line_data) {
          curcol = 1; // reset (starting new line)
          normalized_data = line_data;
          StrUpr(normalized_data); // make line all upcase
          //------------
          // parse words
          //------------
          parsestr = " ,.;:!?()/-*@\\#&";
          word_count = StrParse(original_words,line_data,parsestr);
          word_count = StrParse(upcase_words,normalized_data,parsestr);
          wordix = 0; // first word index
          while (!g_end && wordix < word_count) {
            //-----------
            // check word
            //-----------
            word1 = original_words[wordix];
            word2 = upcase_words[wordix];
            wordlen = strlen(word2);
            if ( wordlen >= 2
              && strisalpha(word2) ) {
              check = TRUE;
              i = 0;
              while (i < g_ignore_count) {
                if (word2 == g_ignore_words[i]) {
                  //---------------------------------------
                  // word is in IGNORE list, don't check it
                  //---------------------------------------
                  check = FALSE;
                  i = g_ignore_count; // end loop
                  }
                else {
                  ++i;
                }
              } // while
              if (check) {
                //--------------------
                // SPELLCHECK THE WORD
                //--------------------
                WordCheck(word1,word2,curcol);
              }
            }
            curcol += wordlen + 1; // maintain for FIND
            ++wordix;     // next word index
          } // while
        }
        ++linenum;        // next line number
      } // while
      if (g_modified) {
        //--------------------------------
        // write dictionary with new words
        //--------------------------------
        if (FileExist(filespec)) {
          backup_spec = StrCompose(filespec,".bak");
          FileCopy(filespec,backup_spec); // backup original
        }
        g_hfile = FileOpen(filespec,"w");
        if (g_hfile) {
          SPFservice("vpersist","SpellFile.FileSpec");
          Msg("Writing modified disctionary, this will take a few seconds.");
          if (a_count) {
            i = 0; while (i < a_count) { WordWrite(a_words[i]); ++i; } }
          if (b_count) {
            i = 0; while (i < b_count) { WordWrite(b_words[i]); ++i; } }
          if (c_count) {
            i = 0; while (i < c_count) { WordWrite(c_words[i]); ++i; } }
          if (d_count) {
            i = 0; while (i < d_count) { WordWrite(d_words[i]); ++i; } }
          if (e_count) {
            i = 0; while (i < e_count) { WordWrite(e_words[i]); ++i; } }
          if (f_count) {
            i = 0; while (i < f_count) { WordWrite(f_words[i]); ++i; } }
          if (g_count) {
            i = 0; while (i < g_count) { WordWrite(g_words[i]); ++i; } }
          if (h_count) {
            i = 0; while (i < h_count) { WordWrite(h_words[i]); ++i; } }
          if (i_count) {
            i = 0; while (i < i_count) { WordWrite(i_words[i]); ++i; } }
          if (j_count) {
            i = 0; while (i < j_count) { WordWrite(j_words[i]); ++i; } }
          if (k_count) {
            i = 0; while (i < k_count) { WordWrite(k_words[i]); ++i; } }
          if (l_count) {
            i = 0; while (i < l_count) { WordWrite(l_words[i]); ++i; } }
          if (m_count) {
            i = 0; while (i < m_count) { WordWrite(m_words[i]); ++i; } }
          if (n_count) {
            i = 0; while (i < n_count) { WordWrite(n_words[i]); ++i; } }
          if (o_count) {
            i = 0; while (i < o_count) { WordWrite(o_words[i]); ++i; } }
          if (p_count) {
            i = 0; while (i < p_count) { WordWrite(p_words[i]); ++i; } }
          if (q_count) {
            i = 0; while (i < q_count) { WordWrite(q_words[i]); ++i; } }
          if (r_count) {
            i = 0; while (i < r_count) { WordWrite(r_words[i]); ++i; } }
          if (s_count) {
            i = 0; while (i < s_count) { WordWrite(s_words[i]); ++i; } }
          if (t_count) {
            i = 0; while (i < t_count) { WordWrite(t_words[i]); ++i; } }
          if (u_count) {
            i = 0; while (i < u_count) { WordWrite(u_words[i]); ++i; } }
          if (v_count) {
            i = 0; while (i < v_count) { WordWrite(v_words[i]); ++i; } }
          if (w_count) {
            i = 0; while (i < w_count) { WordWrite(w_words[i]); ++i; } }
          if (x_count) {
            i = 0; while (i < x_count) { WordWrite(x_words[i]); ++i; } }
          if (y_count) {
            i = 0; while (i < y_count) { WordWrite(y_words[i]); ++i; } }
          if (z_count) {
            i = 0; while (i < z_count) { WordWrite(z_words[i]); ++i; } }
          FileClose(g_hfile);
        }
      }
      msg = strcompose("Spell check complete, dictionary currently contains ",all_count," words.");
      Msg(msg);
    }
    }
  else {
    Msg("Not in Edit, can not continue.");
  }

} // spellchk

//----------
// WordCheck
//----------
void WordCheck(char *original_word, char *upcase_word, int curcol) {

  // This function checks "upcase_word" against all the words in
  // the list of words with the same first character.

  char *BtnPushed;
  char *cmd;
  char *new_word;
  char *first_char;
  char *dialog_rc;
  int   i;
  BOOL  any_error = FALSE;
  BOOL  cancel = FALSE;
  BOOL  found = FALSE;

  first_char = StrGetSubstr(upcase_word,1,1);
  i = 0;
  switch (first_char) {
    case "A":
      while (!found && i < a_count)
      { if (upcase_word == a_words[i]) found = TRUE; else ++i; } break;
    case "B":
      while (!found && i < b_count)
      { if (upcase_word == b_words[i]) found = TRUE; else ++i; } break;
    case "C":
      while (!found && i < c_count)
      { if (upcase_word == c_words[i]) found = TRUE; else ++i; } break;
    case "D":
      while (!found && i < d_count)
      { if (upcase_word == d_words[i]) found = TRUE; else ++i; } break;
    case "E":
      while (!found && i < e_count)
      { if (upcase_word == e_words[i]) found = TRUE; else ++i; } break;
    case "F":
      while (!found && i < f_count)
      { if (upcase_word == f_words[i]) found = TRUE; else ++i; } break;
    case "G":
      while (!found && i < g_count)
      { if (upcase_word == g_words[i]) found = TRUE; else ++i; } break;
    case "H":
      while (!found && i < h_count)
      { if (upcase_word == h_words[i]) found = TRUE; else ++i; } break;
    case "I":
      while (!found && i < i_count)
      { if (upcase_word == i_words[i]) found = TRUE; else ++i; } break;
    case "J":
      while (!found && i < j_count)
      { if (upcase_word == j_words[i]) found = TRUE; else ++i; } break;
    case "K":
      while (!found && i < k_count)
      { if (upcase_word == k_words[i]) found = TRUE; else ++i; } break;
    case "L":
      while (!found && i < l_count)
      { if (upcase_word == l_words[i]) found = TRUE; else ++i; } break;
    case "M":
      while (!found && i < m_count)
      { if (upcase_word == m_words[i]) found = TRUE; else ++i; } break;
    case "N":
      while (!found && i < n_count)
      { if (upcase_word == n_words[i]) found = TRUE; else ++i; } break;
    case "O":
      while (!found && i < o_count)
      { if (upcase_word == o_words[i]) found = TRUE; else ++i; } break;
    case "P":
      while (!found && i < p_count)
      { if (upcase_word == p_words[i]) found = TRUE; else ++i; } break;
    case "Q":
      while (!found && i < q_count)
      { if (upcase_word == q_words[i]) found = TRUE; else ++i; } break;
    case "R":
      while (!found && i < r_count)
      { if (upcase_word == r_words[i]) found = TRUE; else ++i; } break;
    case "S":
      while (!found && i < s_count)
      { if (upcase_word == s_words[i]) found = TRUE; else ++i; } break;
    case "T":
      while (!found && i < t_count)
      { if (upcase_word == t_words[i]) found = TRUE; else ++i; } break;
    case "U":
      while (!found && i < u_count)
      { if (upcase_word == u_words[i]) found = TRUE; else ++i; } break;
    case "V":
      while (!found && i < v_count)
      { if (upcase_word == v_words[i]) found = TRUE; else ++i; } break;
    case "W":
      while (!found && i < w_count)
      { if (upcase_word == w_words[i]) found = TRUE; else ++i; } break;
    case "X":
      while (!found && i < x_count)
      { if (upcase_word == x_words[i]) found = TRUE; else ++i; } break;
    case "Y":
      while (!found && i < y_count)
      { if (upcase_word == y_words[i]) found = TRUE; else ++i; } break;
    case "Z":
      while (!found && i < z_count)
      { if (upcase_word == z_words[i]) found = TRUE; else ++i; } break;
  } // switch
  if (!found) {
    //-------------------------
    // highlight word not found
    //-------------------------
    cmd = StrCompose("find c'",original_word,"'");
    SPFservice("set","col_number",curcol);
    SPFservice("cmd",cmd);
    //--------------------
    // present word dialog
    //--------------------
    SPFservice("vput","SpellCheck.Word",original_word);
    dialog_rc = SPFservice("dialog","SpellCheck");
    switch (dialog_rc) {

      case "OK":
        BtnPushed = SPFservice("vget","SpellCheck.BtnPushed");
        break;

      case "HELP":
        cancel = TRUE;
        Msg("HELP not available for this dialog.");
        break;

      case "CANCEL":
        cancel = TRUE;
        break;

      case "ERROR":
        any_error = TRUE;
        break;

    } // switch
    if (cancel) {
      g_end = TRUE;
    }
    if (!any_error && !cancel) {
      switch (BtnPushed) {

        case "Change":
          //--------------------------------------
          // change specific instance of this word
          //--------------------------------------
          new_word = SPFservice("vget","SpellCheck.Word");
          cmd = StrCompose("change ",original_word," ",new_word);
          SPFservice("cmd",cmd);
          break;

        case "ChangeAll":
          //----------------------------------
          // change all instances of this word
          //----------------------------------
          new_word = SPFservice("vget","SpellCheck.Word");
          cmd = StrCompose("change all ",original_word," ",new_word);
          SPFservice("cmd",cmd);
          break;

        case "Ignore":
          //--------------------------------------
          // ignore specific instance of this word
          //--------------------------------------
          // nothing to do, continue
          break;

        case "IgnoreAll":
          //----------------------------------
          // ignore all instances of this word
          //----------------------------------
          g_ignore_words[g_ignore_count] = upcase_word;
          ++g_ignore_count;
          break;

        case "Add":
          //-----------------------
          // add word to dictionary
          //-----------------------
          ++all_count; // incr total word count
          switch (first_char) {
            case "A": a_words[a_count] = upcase_word; ++a_count; break;
            case "B": b_words[b_count] = upcase_word; ++b_count; break;
            case "C": c_words[c_count] = upcase_word; ++c_count; break;
            case "D": d_words[d_count] = upcase_word; ++d_count; break;
            case "E": e_words[e_count] = upcase_word; ++e_count; break;
            case "F": f_words[f_count] = upcase_word; ++f_count; break;
            case "G": g_words[g_count] = upcase_word; ++g_count; break;
            case "H": h_words[h_count] = upcase_word; ++h_count; break;
            case "I": i_words[i_count] = upcase_word; ++i_count; break;
            case "J": j_words[j_count] = upcase_word; ++j_count; break;
            case "K": k_words[k_count] = upcase_word; ++k_count; break;
            case "L": l_words[l_count] = upcase_word; ++l_count; break;
            case "M": m_words[m_count] = upcase_word; ++m_count; break;
            case "N": n_words[n_count] = upcase_word; ++n_count; break;
            case "O": o_words[o_count] = upcase_word; ++o_count; break;
            case "P": p_words[p_count] = upcase_word; ++p_count; break;
            case "Q": q_words[q_count] = upcase_word; ++q_count; break;
            case "R": r_words[r_count] = upcase_word; ++r_count; break;
            case "S": s_words[s_count] = upcase_word; ++s_count; break;
            case "T": t_words[t_count] = upcase_word; ++t_count; break;
            case "U": u_words[u_count] = upcase_word; ++u_count; break;
            case "V": v_words[v_count] = upcase_word; ++v_count; break;
            case "W": w_words[w_count] = upcase_word; ++w_count; break;
            case "X": x_words[x_count] = upcase_word; ++x_count; break;
            case "Y": y_words[y_count] = upcase_word; ++y_count; break;
            case "Z": z_words[z_count] = upcase_word; ++z_count; break;
          } // switch
          g_modified = TRUE;
          break;

      } // switch
    }
  }

} // WordCheck

//----------
// WordWrite
//----------
void WordWrite(char *word) {

  FileRecWrite(g_hfile,word);

} // WordWrite
