/*************************************************************************
**                                                                      **
**           (c) Copyright 1999-2003, Tachyon Software LLC.             **
**                                                                      **
**  FILE:          TXA.C                                                **
**                                                                      **
**  PURPOSE:       Edit macro for SPF/Source Edit to invoke the         **
**                 Tachyon z/Assembler on the program currently         **
**                 being edited and to display any resulting            **
**                 errors as note lines.                                **
**                                                                      **
**  REQUIREMENTS:  SPF/Source Edit 2.5 Build 269 or                     **
**                 SPF/Source Edit 3.0 Build 454 is required.           **
**                                                                      **
**                 The "Macro Source" path under Options...Paths...     **
**                 must include the directory containing this file.     **
**                                                                      **
**                 The "Edit Write Lock" option should be disabled      **
**                 under Options...Edit...                              **
**                                                                      **
**                 If the Tachyon z/Assembler is not installed          **
**                 in the default directory C:\Tachyon, follow the      **
**                 directions in the comment later in this file so      **
**                 this macro can find the assembler program.           **
**                                                                      **
**  CHANGES:       1999-03-29: Created.                                 **
**                 2000-05-22: SPF/SE 3.0 build 454 support             **
**                 2001-03-25: SPF/SE 3.5 build 572 support             **
**                 2003-05-15: SPF/SE 4.0 support                       **
**                                                                      **
*************************************************************************/

void TXA (
    int    argc,
    char * argv[])
{
    /**********************************************************
    **  If the Tachyon z/Assembler is not installed in       **
    **  C:\Tachyon, either set the X390DIR environment       **
    **  variable or change the next statement.               **
    **********************************************************/
    char * x390 = "C:\\Tachyon";
    char * suspend = "";
    char * outmsg;
    char * outerr;
    char * ret;
    char * str;
    char * source;
    char * lineno;
    char * syslib;
    char * cur_package;
    char * base_copy;
    char * mvs_macs;
    char * sernet_cpy;
    int    file;
    int    pos;
    int    len;
    int    lines = 0;

    strcpy(base_copy, "c:\\ken\\work\\kencpylib\\*.cpy");
    strcpy(mvs_macs, "c:\\ken\\work\\maclib\\*.mac");
    strcpy(sernet_cpy, "c:\\ken\\work\\svncpylib\\*.cpy");

    /**********************************************************
    **  Determine if the "suspend" option is needed for the  **
    **  "system" command.  "suspend" is not needed or        **
    **  supported in SPF/SE 2.5.  By default, in SPF/SE 3.0  **
    **  the "system" command does not wait for the invoked   **
    **  command to complete.  In build 451, support for the  **
    **  "suspend" option was added to force the "system"     **
    **  command to act like it did in SPF/SE 2.5.  However,  **
    **  not until SPF/SE 3.0 build 454 was a way added to    **
    **  determine what SPF/SE release was running, and thus  **
    **  if the "suspend" option is needed.  Then in SPF/SE   **
    **  4.0, the "suspend" option was dropped again.         **
    **********************************************************/
    ret = SPFservice("vget", "ZSPFVERSION", NULL);
    if (ret != "6.0")         // *** TRT 05/04/07 ***
        suspend = "suspend ";

    /***********************************************************
    **  Determine the names of the SYSTERM and stderr files.  **
    ***********************************************************/
    outmsg = getenv("TMP");
    if (outmsg == NULL || strlen(outmsg) == 0)
        outmsg = getenv("TEMP");
    if (outmsg == NULL || strlen(outmsg) == 0)
        outmsg = "C:";
    else strstriptrailing(outmsg, "\\");
    outerr = getenv("X390ERR");
    if (outerr == NULL || strlen(outerr) == 0) {
        /*********************************************************
        **  Unlike Windows NT, the command interpreter of       **
        **  Windows 95 and Windows 98 and Windows ME is so      **
        **  brain-dead that it can't redirect stderr using the  **
        **  "2>filename" parameter.  To support Windows 95 and  **
        **  Windows 98, the assembler recognizes the X390ERR    **
        **  environment variable as a way to redirect stderr.   **
        **  Until everyone switches to Windows XP, the          **
        **  X390ERR enviroment variable must be set.            **
        ***********************************************************/
        strcpy(outerr, outmsg);
        strcat(outerr, "\\TXAERR.TXT");
        putenv("X390ERR", outerr);
    }
    strcat(outmsg, "\\TXAMSG.TXT");

    /***************************************
    **  If needed, save the source code.  **
    ***************************************/
    source = SPFservice("query", "FILESPEC", NULL);
    ret  = SPFservice("query", "FILEMODIFIED", NULL);
    if (strcmp(ret, "TRUE") == 0) {
        SPFservice("cmd", "SAVE", NULL);
    }
    SPFservice("cmd", "RESET SPECIAL", NULL);

    /****************************
    **  Invoke the assembler.  **
    ****************************/
    ret = getenv("X390DIR");
    if (ret != NULL && strlen(ret) != 0)
        x390 = ret;
    strstriptrailing(x390, "\\");
    syslib = strcompose(" \"-SL" ,cur_package, ";", base_copy, ";", mvs_macs, ";", sernet_cpy, "\"");
    x390 = strcompose(x390, "\\x390.exe -ML(2,2) -NO -NAD -SZMAX -TB ", syslib, " -SM", outmsg, " \"", source, "\"");
    str = strcompose("system nointerp ", suspend, x390);
    filedelete(outmsg);
    filedelete(outerr);
    SPFservice("cmd", str, NULL);

    /**************************************************************************
    **  Any fatal error messages are inserted before the first source line.  **
    **************************************************************************/
    file = fileopen(outerr, "r");
    if (file == 0) {
        /***********************************************************
        **  Something went really wrong - probably the assembler  **
        **  was not found.  Here we try to rexecute it with the   **
        **  "stop" option so the user can see what happened.      **
        ***********************************************************/
        putenv("X390ERR", NULL);
        pos = strfindsubstr(x390, "-Q ", 1);
        strdelsubstr(x390, pos, 3);
        str = strcompose("system interp stop ", suspend, x390);
        SPFservice("cmd", str, NULL);
        file = fileopen(outerr, "r");
    }
    if (file != 0) {
        ret = filerecread(file, str);
        while (strcmp(ret, "OK") == 0) {
            ++lines;
            SPFservice("cmd", "LOCATE 1 1", NULL);
            str = strcompose("NOTELINE ", str);
            SPFservice("cmd", str, NULL);
            ret = filerecread(file, str);
        }
        fileclose(file);
        filedelete(outerr);
        file = fileopen(outmsg, "r");
    }
    /**************************************************************************
    **  Any assembly messages are inserted before the source line in error.  **
    **************************************************************************/
    if (file != 0) {
        ret = filerecread(file, str);
        while (strcmp(ret, "OK") == 0) {
            pos = strfindsubstr(str, ") : ", 1);
            if (pos == 0) {
                lineno = "1";
            } else {
                lineno = strgetsubstr(str, 1, pos);
                pos += 3;
                strdelsubstr(str, 1, pos);
                pos = strfindlast(lineno, "(");
                strdelsubstr(lineno, 1, pos);
                strstriptrailing(lineno, ")");
            }
            lineno = strcompose("LOCATE ", lineno, " 1");
            ++lines;
            SPFservice("cmd", lineno, NULL);
            str = strcompose("NOTELINE ", str);
            SPFservice("cmd", str, NULL);

            ret = filerecread(file, str);
        }
        fileclose(file);
        filedelete(outmsg);
    }
    if (lines != 0) {
        SPFservice("cmd", "LOCATE 0", NULL);
        SPFservice("cmd", "LOCATE SPECIAL", NULL);
        if (lines == 1)
            str = "1 assembly message";
        else str = strcompose(lines, " assembly messages");
        msg(str);
    }
}
